-- Author	  : admin
-- Create Date : 9/16/2020 5:12:50 PM

local addonName, addon = ...;
local module = addon:NewModule('CraftUI', 'AceEvent-3.0');

module.professionName = 'UNKNOWN';
module.GetNumItems = 'GetNumCrafts';
module.GetItemID = 'GetCraftID';
module.GetItemInfo = 'GetCraftInfo';
module.GetItemLink = 'GetCraftItemLink';
module.GetItemNumReagents = 'GetCraftNumReagents';
module.GetItemReagentInfo = 'GetCraftReagentInfo';
module.ExpandItemHeader = 'ExpandCraftSkillLine';
module.CollapseItemHeader = 'CollapseCraftSkillLine';

local GetCraftInfo              = _G.GetCraftInfo
local GetCraftItemLink          = _G.GetCraftItemLink
local GetCraftNumReagents       = _G.GetCraftNumReagents;
local GetCraftReagentItemLink   = _G.GetCraftReagentItemLink;
local GetCraftSelectionIndex    = _G.GetCraftSelectionIndex;
local GetNumCrafts              = _G.GetNumCrafts;

function module:OnInitialize()
	module:SetEnabledState(false);
end

function module:OnEnable()
	addon:initFrames(CraftFrame, module);

   	module:RegisterEvent('CRAFT_UPDATE');
	CraftFrame:UnregisterEvent('CRAFT_UPDATE');

    for i=1, CRAFTS_DISPLAYED, 1 do
        _G['Craft' .. i]:SetAttribute('id', i);
    end

    addon:RawHook('CraftFrame_SetSelection', module.CraftFrame_SetSelection, true);

    hooksecurefunc('Craft_UpdateTrainingPoints', module.Craft_UpdateTrainingPoints);
end

--
-- methodes
--

function module:getItem(button)
    local index = FauxScrollFrame_GetOffset(CraftListScrollFrame) + button:GetAttribute('id');

    return addon:getSkillData(module.professionName).items[index] or {index = 0};
end

function module:isCraftVisible(selectedIndex)
    local data = addon:getSkillData(module.professionName);
    local selectedIndex = GetCraftSelectionIndex();

    -- correct selection
    for listIndex, item in ipairs(data.items) do
        if (item.index == selectedIndex and listIndex <= data.numItems) then
            return true
        end
    end

    return false;
end


function module:updateFrame()
	securecall(CraftFrame_OnEvent, CraftFrame, 'CRAFT_UPDATE');
end


function GetCraftID(index)
    local id            = module.professionName .. '-';
    local numReagents   = GetCraftNumReagents(index);
    local link          = GetCraftItemLink(index);
    local info          = { GetCraftInfo(index) };

    if (info[3] == 'header') then
        return nil;
    end

    if (link) then
        id = id .. link:match(':(%d+)') .. ':';

        for i = 1, numReagents, 1 do
            id = id .. '-' .. GetCraftReagentItemLink(index, i):match(':(%d+)');
        end

        return id;
    end

    if (info[1] and info[2]) then
        return id .. info[1] .. ':' .. info[2];
    end
end

--
-- Events
--

function module:CRAFT_UPDATE(...)
	if (addon.scanning ~= nil) then
		return;
	end

	addon.scanning = module;
	module.professionName = GetCraftName() or 'UNKNOWN';

	addon:initDB(module.professionName);
	addon:updateButtons();

	addon:scan()
	addon.scanning = nil;

	securecall(CraftFrame_OnEvent, CraftFrame, 'CRAFT_UPDATE');
end


--
-- Hooks
--

function module.Craft_UpdateTrainingPoints()
    if (module.updating) then
        return;
    end
    
    module.updating = true;
    local data = addon:getSkillData(module.professionName);

    CraftHighlightFrame:Hide();

    for i = 1, CRAFTS_DISPLAYED, 1 do
        local button = getglobal("Craft"..i);
        local craftButtonCost = getglobal("Craft"..i..'Cost');

        button.craftItem = module:getItem(button);
        button:SetID(button.craftItem.index);
        
        if (button:IsShown() and GetCraftSelectionIndex() == button.craftItem.index) then
            -- Place the highlight and lock the highlight state
            CraftHighlightFrame:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0);
            CraftHighlightFrame:Show();
            Craft_SetSubTextColor(button, 1.0, 1.0, 1.0);
            craftButtonCost:SetTextColor(1.0, 1.0, 1.0);
            button:LockHighlight();
        else
            button:UnlockHighlight();
        end        
    end

    module.updating = nil;
end

function module:GetNumCrafts()
    if (GetNumCrafts() == 0) then
        -- fix possibble call when craft is closed but an update was triggered by 'SKILL_LINES_CHANGED' event
        return 0;
    end

	return addon:getSkillData(module.professionName).numItems
end

function module.CraftFrame_SetSelection(index)
    addon:Unhook('GetNumCrafts');
    addon:Unhook('GetCraftInfo');

    if (module:isCraftVisible(index) == false) then
        local data = addon:getSkillData(module.professionName);
        index = nil;

        for i, item in ipairs(data.items) do
            if (item.index > 0 and item.items == nil and i <= data.numItems) then
                index = item.index;
                break;
            end
        end
    end

    addon.hooks.CraftFrame_SetSelection(index);

    addon:createGenericHook('GetNumCrafts');
    addon:createGenericHook('GetCraftInfo');
end

