-- Author	  : admin
-- Create Date : 9/27/2019 6:04:14 PM

local addonName, addon = ...;
local module = addon:NewModule('TradeSkillUI');

local _G = _G;

module.professionName = 'UNKNOWN';
module.GetItemID = 'GetTradeSkillID';
module.GetNumItems = 'GetNumTradeSkills';
module.GetItemInfo = 'GetTradeSkillInfo';
module.GetItemLink = 'GetTradeSkillItemLink';
module.GetItemNumReagents = 'GetTradeSkillNumReagents';
module.GetItemReagentInfo = 'GetTradeSkillReagentInfo';
module.ExpandItemHeader = 'ExpandTradeSkillSubClass';
module.CollapseItemHeader = 'CollapseTradeSkillSubClass';

local GetTradeSkillNumReagents      = _G.GetTradeSkillNumReagents;
local GetTradeSkillInfo             = _G.GetTradeSkillInfo;
local GetTradeSkillItemLink         = _G.GetTradeSkillItemLink;
local GetTradeSkillReagentItemLink  = _G.GetTradeSkillReagentItemLink;

function module:OnEnable()
	frame = addon:initFrames(TradeSkillFrame, module);

	module:hookTooltips();

	addon:RegisterEvent('TRADE_SKILL_UPDATE', module.TRADE_SKILL_UPDATE);
	TradeSkillFrame:UnregisterEvent('TRADE_SKILL_UPDATE');
end

function module:OnDisable()
	addon:UnregisterAllEvents();

	TradeSkillFrame:RegisterEvent('TRADE_SKILL_UPDATE');
end

--
-- Module methodes
--

function module:updateFrame()
	securecall(TradeSkillFrame_OnEvent, TradeSkillFrame, 'TRADE_SKILL_UPDATE');
end

function GetTradeSkillID(index)
    local id            = module.professionName .. '-';
    local numReagents   = GetTradeSkillNumReagents(index);
    local link          = GetTradeSkillItemLink(index);
    local info          = { GetTradeSkillInfo(index) };

    if (info[2] == 'header') then
        return nil;
    end

    if (link) then
        id = id .. link:match(':(%d+)') .. ':';

        for i = 1, numReagents, 1 do
            id = id .. '-' .. GetTradeSkillReagentItemLink(index, i):match(':(%d+)');
        end

        return id;
    end
end

--
-- Events
--

function module.TRADE_SKILL_UPDATE(...)
	if (addon.scanning ~= nil) then
		return;
	end

	addon.scanning = module;
	module.professionName = GetTradeSkillLine();

	addon:initDB(module.professionName);
	addon:updateButtons();

	addon:scan()
	addon.scanning = nil;

	securecall(TradeSkillFrame:GetScript('OnEvent'), TradeSkillFrame, ...);
end

--
-- Hooks
--

function module:GetFirstTradeSkill()
	local data = addon:getSkillData(module.professionName).items;

	for i, item in ipairs(data) do
		if(item.items == nil and item.index > 0) then
			return i;
		end
	end

	return 0;
end

function module:GetNumTradeSkills()
	return addon:getSkillData(module.professionName).numItems
end

function module:GetTradeSkillSelectionIndex()
	local index = addon.hooks.GetTradeSkillSelectionIndex();

	local data = addon:getSkillData(module.professionName).items;

	for i, item in ipairs(data) do
		if(item.index == index and item.index > 0) then
			return i;
		end
	end

	return 0;
end

function module:hookTooltips()
	newArray(TradeSkillDetailScrollChildFrame:GetChildren())
		.forEach(function(icon)
			icon.UpdateTooltip = module.UpdateTooltip;
			icon:SetScript('OnEnter', module.UpdateTooltip);
		end);
end

function module.UpdateTooltip(self, mouseOver)
	local index, link = addon:getRealIndex(TradeSkillFrame.selectedSkill, module.professionName);

	if (self:GetID() > 0) then
		link = addon.hooks.GetTradeSkillReagentItemLink(index, self:GetID());
	else
		link = addon.hooks.GetTradeSkillItemLink(index);
	end

	GameTooltip:Hide();
	GameTooltip:SetOwner(self, 'ANCHOR_RIGHT');
	GameTooltip:SetHyperlink(link);
	GameTooltip:Show();

	CursorUpdate(self);
end
